#include <Adafruit_CircuitPlayground.h>
#include <Wire.h>
#include <SPI.h>
#define TINY_GSM_MODEM_ESP8266
#include "pitches.h"
#define SerialMon Serial
#define SerialAT Serial1

//Adafruit account details
#define MQTT_STUDENT_USERNAME ""
#define MQTT_USERNAME ""
#define MQTT_PASSWORD "<io_key>"

const char MQTT_CLIENTID[] = MQTT_STUDENT_USERNAME __DATE__ __TIME__;

// Your WiFi connection credentials, if applicable
const char wifiSSID[] = "";
const char wifiPass[] = "";

// MQTT details
const char* broker = "io.adafruit.com";
const char questionsTopic[] = MQTT_USERNAME "/feeds/office-hour.questions";
const char studentsTopic[] = MQTT_USERNAME "/feeds/office-hour.students";
const char instructorTopic[] = MQTT_USERNAME "/feeds/office-hour.instructor";


#include <TinyGsmClient.h>
#include <PubSubClient.h>

//Create MQTT client
TinyGsm modem(SerialAT);
TinyGsmClient client(modem);
PubSubClient mqtt(client);

unsigned long lightsOn = 0; //Holds the millis() millisecond of when the neopixel led where last turned on so we can turn them off after 10 seconds.
uint32_t lastReconnectAttempt = 0; ////Holds the millis() millisecond of the last reconnection attempt to the mqtt server

//Happy song
int melody[] = {
  NOTE_FS5, NOTE_FS5, NOTE_D5, NOTE_B4, NOTE_B4, NOTE_E5,
  NOTE_E5, NOTE_E5, NOTE_GS5, NOTE_GS5, NOTE_A5, NOTE_B5,
  NOTE_A5, NOTE_A5, NOTE_A5, NOTE_E5, NOTE_D5, NOTE_FS5,
  NOTE_FS5, NOTE_FS5, NOTE_E5, NOTE_E5, NOTE_FS5, NOTE_E5
};
// The notes duration.
int durations[] = {
  8, 8, 8, 4, 4, 4,
  4, 5, 8, 8, 8, 8,
  8, 8, 8, 4, 4, 4,
  4, 5, 8, 8, 8, 8
};
int songLength = sizeof(melody) / sizeof(melody[0]);
float soundValue; //Will hold the sound value received from the sound sensor

void setup() {
  CircuitPlayground.begin();


  SerialMon.begin(115200);
  delay(1000);
  SerialAT.begin(115200);
  SerialMon.println("Initializing modem...");
  modem.restart();

  String modemInfo = modem.getModemInfo();
  SerialMon.print("Modem Info: ");
  SerialMon.println(modemInfo);

  SerialMon.print(F("Setting SSID/password..."));
  if (!modem.networkConnect(wifiSSID, wifiPass)) {
    SerialMon.println(" fail");
    while (true);
  }
  SerialMon.println(" success");

  SerialMon.print("Waiting for network...");
  if (!modem.waitForNetwork()) {
    SerialMon.println(" fail");
    while (true);
  }
  SerialMon.println(" success");

  if (modem.isNetworkConnected()) {
    SerialMon.println("Network connected");
  }

  // MQTT Broker setup
  mqtt.setServer(broker, 1883);
  mqtt.setCallback(mqttCallback);

}

void loop() {
  if (!mqtt.connected()) {
    uint32_t t = millis();
    if (t - lastReconnectAttempt > 10000L) {
      lastReconnectAttempt = t;
      if (mqttConnect()) {
        lastReconnectAttempt = 0;
      }
    }
    delay(100);
    return;
  }
  mqtt.loop();

  // disconnects occur otherwise
  delay(1000);

  //send hand claps to the instructor if the student made a loud sound.
  soundValue = CircuitPlayground.mic.soundPressureLevel(10);
  if (soundValue > 80) {
    sendClapsToInstructor();
  }
  //Turn off the neopixel lights 5 seconds after they were turned on.
  if (millis() - lightsOn > 50000) {
    CircuitPlayground.clearPixels();
  }

}

//This function is called every time data is published on on eof the topics that the client has subscribed to.
void mqttCallback(char* topic, byte* payload, unsigned int len) {
  SerialMon.print("Message arrived [");
  SerialMon.print(topic);
  SerialMon.print("]: ");
  SerialMon.write(payload, len);
  SerialMon.println();

  // Only proceed if incoming messages topic matches to the instructor topic which means the students got a message from the instructor.
  if (String(topic) == instructorTopic) {
    // parse incoming number.
    int msg_id = (int)payload[0] - '0';
    if (msg_id == 1) {
      //This message means that the office hour started, so we light the leds in green to signal to the students that they should connect to zoom.
      blink_lights(0, 255, 0, 10);
    }
    else if (msg_id == 2) {
      //This message means that there is a break , so we light the leds in red to signal to the students that they can disconnect from zoom.
      blink_lights(255, 0, 0, 10);
    }
    else if (msg_id == 3) {
      //This message means that the break is over , so we play a tune to signal to the students that they should connect to zoom.
      play_melody();
    }
  }
  // Only proceed if incoming message's topic matches to the questuions topic which means the instructor is moving to the recieved question.
  else if (String(topic) == questionsTopic) {
    // parse incoming number.
    int question_id = (int)payload[0] - '0';
    //We light the neopixels lights in purple accrding to the question that is about to be reviewed.
    blink_lights(149, 53, 83, question_id);
  }
}

//This function connects to the MQTT server.
boolean mqttConnect() {
  SerialMon.print("Connecting to ");
  SerialMon.print(broker);

  boolean status = mqtt.connect(MQTT_CLIENTID, MQTT_USERNAME, MQTT_PASSWORD);
  if (status == false) {
    SerialMon.println(" fail");
    return false;
  }
  SerialMon.println(" success");
  //Subscribe to the needed feeds(topics)
  mqtt.subscribe(questionsTopic);
  mqtt.subscribe(instructorTopic);

  return mqtt.connected();
}

//This function is used to turn on the neopixel lights.
void blink_lights(int r, int g, int b, int ledNum) {
  for (int i = 0 ; i < ledNum; ++i) {
    CircuitPlayground.setPixelColor(i, r,   g,   b);
  }
  lightsOn = millis(); //save the time of the neopixles lighting
}

//This function is used to turn off the neopixel leds.
void turnNeopixelsOff() {
  for (int i = 0 ; i < 10; ++i) {
    CircuitPlayground.setPixelColor(1, 0,   0,   0);
  }
  lightsOn = 0;
}

//This function is used to send hand claps to the instructor when the student taps twice on the circuit.
void sendClapsToInstructor(void) {
  mqtt.publish(studentsTopic, "2");
}

//play a melody which is taken from: https://create.arduino.cc/projecthub/GeneralSpud/passive-buzzer-song-take-on-me-by-a-ha-0f04a8
void play_melody() {
  for (int thisNote = 0; thisNote < songLength; thisNote++) {
    // to calculate the note duration, take one second divided by the note type.
    //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000 / durations[thisNote];
    CircuitPlayground.playTone(melody[thisNote], noteDuration);

    // to distinguish the notes, set a minimum time between them.
    // the note's duration + 30% seems to work well:
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
  }
}
